#!/bin/bash
# Author: Erling Preben Hansen erling@eph.dk
#If you find errors please inform me by mail
#
# Do not edit ANYTHING in this file. Unless you know what you are doing.
# Run install.sh as root or with sudo. Answer the questions according to your servers install.
# And you should be good to do.
# Command to run this file....
# sudo bash install.sh
#
# TO DO: Just read Google's "Shell Style Guide". Obviously still a lot todo.
# Known variables here
declare -a new_themes=(Red-wine )
declare -a module=(agora ansel gollem imp ingo kronolith mnemo nag passwd sesha trean turba wicked whups )

# Server specific variables
hordepath=""		# The path to Horde directories
webowner=""		# The user for Horde directories.
webgrp=""		# The group that webserver is run as.
permdir=""		# Access permissions for directories.
permfile=""		# Access permissions for files.

# Do we know where we are.
thispath=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cppath="$thispath/horde"
# Ask and set needed variables
echo -en "\e[91;40m Type the path to your horde install without a trailing slash: \e[0m"
read hordepath
echo -en "\e[91;40m Enter the username for Horde directories and files: \e[0m"
read webowner
echo -en "\e[91;40m Enter the usergroup for Horde directories and files: \e[0m"
read webgrp
echo -e "\e[91;40m Enter permissions for Horde directories: \e[0m"
echo -en "\e[91;40m in the numbered form as in 550. Default is 550: \e[0m"
read permdir
# Set default if nothing is entered
if [ -z "${permdir}" ]; then permdir="550" ; fi
echo -e "\e[91;40m Enter permissions for Horde files: \e[0m"
echo -en "\e[91;40m in numbered form as in 440. Default is 440: \e[0m"
read permfile
# Set default id nothing is entered
if [ -z "${permfile}" ]; then permfile="440" ; fi

# Find installed themes and secure default theme.
declare -a all_themes=($(find ${hordepath}/themes/ -maxdepth 1 -type d -printf '%P\n'))
declare -a installed_themes=(${all_themes[@]/default/} )
# Placeholder for themes we care for and can upgrade.
declare -a upgrade_themes=()

# thanks to various linux forums
function containsElement () {
  local e
  for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
  return 1
}

function interfering {
  count_ins=${#installed_themes[@]}
  for ((y=0; y<$count_ins; y++ ))
  do
    ins_theme=${installed_themes[$y]}
    containsElement "${ins_theme,,}" "${new_themes[@],,}"
    if [ $? == 0 ]
    then
      upgrade_themes=("${upgrade_themes[@]}" "${ins_theme}")
    fi
  done
}

function remove {

# Chech if theme is present, if it is delete theme, else warn.
# We delete old themes of ours because the size of graphics/ is very different, at this time.
  if [ -d "$hordepath/$themepath" ]
  then
    rm -r $hordepath/$themepath
    count=${#module[@]}
    for ((c=0; c<$count; c++ ))
    do
      modname=${module[$c]}
      if [ -d $hordepath/$modname/$themepath ]
      then
	rm -r $hordepath/$modname/$themepath
      else
	echo "info: Nothing to upgrade in ${modname} ${themename}" >> ${thispath}/install.log
      fi
    done
  else
    echo -e "\e[33;40m          ! We have an internal error ! \e[0m"
    echo -e "\e[33;40m You may have to remove the old themes manually! \e[0m"
  fi
}

function upgrade {
  if [ -d $hordepath/themes/ ]
  then
    count_upgr_themes=${#upgrade_themes[@]}
    for ((x=0; x<$count_upgr_themes; x++ ))
    do
      themename=${upgrade_themes[$x]}
      themepath="/themes/${themename}"
      remove
    done
    echo -e "\e[91;40m Is ready to install your new horde theme \e[0m"
  else
    echo -e "\e[33;40m The path to your Horde install is wrong. \e[0m"
    echo -e "\e[33;40m Please correct this and try again. \e[0m"
    Exit 1
  fi
}

function copytodest {
  cp -v -r $cppath$themepath $hordepath$themepath >> ${thispath}/install.log
  count=${#module[@]}
  for ((c=0; c<$count; c++ ))
  do
    modname=${module[$c]}
    if [[ -d $cppath/$modname$themepath && -d $hordepath/$modname ]]
    then
      cp -v -r $cppath/$modname$themepath $hordepath/$modname$themepath >> ${thispath}/install.log
    else
      echo "info: Nothing to install for ${modname} ${themename}." >> ${thispath}/install.log
    fi
  done
}

function setown {
  chown -R $webowner:$webgrp $hordepath/$themepath
  count=${#module[@]}
  for ((c=0; c<$count; c++ ))
  do
    modname=${module[$c]}
    if [ -d $hordepath/$modname$themepath ]
    then
      chown -R $webowner:$webgrp $hordepath/$modname$themepath
    fi
  done
}

function setperm {
  find $hordepath$themepath -type d -exec chmod $permdir {} \;
  find $hordepath$themepath -type f -exec chmod $permfile {} \;
  count=${#module[@]}
  for ((c=0; c<$count; c++ ))
  do
    modname=${module[$c]}
    if [ -d $hordepath/$modname$themepath ]
    then
      find $hordepath/$modname$themepath -type d -exec chmod $permdir {} \;
      find $hordepath/$modname$themepath -type f -exec chmod $permfile {} \;
    fi
  done
}

function setpermxarg {
  find $hordepath$themepath -type d -print0 | xargs -0 -I dir chmod 750 dir
  find $hordepath$themepat -type f -print0 | xargs -0 -I file chmod 640 file
  count=${#module[@]}
  for ((c=0; c<$count; c++ ))
  do
    modname=${module[$c]}
    if [ -d $hordepath/$modname$themepath ]
    then
      find $hordepath/$modname$themepath -type d -print0 | xargs -0 -I dir chmod 750 dir
      find $hordepath/$modname$themepath -type f -print0 | xargs -0 -I file chmod 640 file
    fi
  done
}

function install_themes {
# Check if hordepath is set correct if not exit and warn.
  if [ -d $hordepath/themes/ ]
  then
    count_newTh=${#new_themes[@]}
    for ((x=0; x<$count_newTh; x++ )); do
      themename=${new_themes[$x]}
      themepath="/themes/${themename}"
      echo "installing the ${themename} theme"
	if [ -d "$cppath$themepath" ]
	then
	  copytodest
	  setown
	  setperm

#If you feel more safe in using xargs instead of "find -exec" you can uncomment the next line
# and comment out the line above.

	  #setpermxarg

	  echo "Done"

	else
	  echo -e "\e[33;40m Warn: Can't find theme $themename."
	  echo -e " $cppath$themepath"
	  echo -e " Did you move this file away from its original placement? \e[0m"
	  exit 1
	fi
    done
  else
    echo -e "\e[33;40m Warn: The path to your Horde install is wrong. Please try again. \e[0m"
    exit 1
  fi
}
#-----------------------------
interfering

if [ ${#upgrade_themes[@]} -eq 0 ]; then
  install_themes
else
  echo -e "\e[91;40m It seems you have the following interfering theme installed \e[0m"
  echo "${upgrade_themes[@]}"
  echo -en "\e[91;40m Do you want to upgrade it? Yes or No. \e[0m"
  read remove
    case "${remove^^}" in
      "Y" | "YES" ) echo -e "\e[91;40m Upgrading the ${upgrade_themes[@]} theme in ${hordepath} \e[0m"
	upgrade
	install_themes
      ;;
      "N" | "NO" ) echo -e "\e[33;40m Exit because of posible name mixup of old and new theme \e[0m"
      ;;
      * ) echo -e "\e[33;40m You need to answer either y, yes, or n, no, to this. \e[0m"
      ;;
    esac
fi
