#!/bin/bsash

# Bash script to remove a theme from Horde
# Author Erling Preben Hansen erling@eph.dk
# You may use distribute and edit this script as you wish

# Define The path to your Horde install.
# If no path is written on commandline this one will be used.
# Edit this or execute with "bash remove-theme.sh /path/to/hordedir"
hordepath=""

# Define the name of the theme you wan't to remove.
themename=""

# You don't need to tamper with this
themepath=""

# Define The Horde apps where this theme may be present.
# don't worry if some of these apps isn't present. 
# You will just get a Directory does not exist from rm on your cmd line. 
declare -a module=(agora ansel gollem imp ingo kronolith mnemo nag passwd sesha trean turba wicked whups )
declare -a our_themes=(Gold Matrix Red-wine White XP-blue )

# Check if a path is written on cmd.
if [ -z $1 ]; then
  echo -en "\e[91;40m Write the path to your Horde install whitout any trailing slash:  \e[0m"
  read hordepath
else
  hordepath="$1"
  echo -e "\e[91;40m Looking for horde themes in $hordepath \e[0m"
fi

declare -a all_themes=($(find ${hordepath}/themes/ -maxdepth 1 -type d -printf '%P\n'))
declare -a installed_themes=(${all_themes[@]/default/} )
# Placeholder for themes we care for and can upgrade.
declare -a upgrade_themes=()

function remove {
# Check if hordepath is set correct, if not exit and warn, if it is start removal.
  if [ -d "${hordepath}/themes" ];  then
# Chech if theme is present, if delete theme in horde and installed apps, else warn.
    if [ -d "${hordepath}/${themepath}" ]; then
      rm -r "${hordepath}/${themepath}"
      count="${#module[@]}"
      for ((c=0; c<"${count}"; c++ )); do
	modname="${module[$c]}"
	  if [ -d "${hordepath}/${modname}${themepath}" ]; then
	    rm -r "${hordepath}/${modname}${themepath}"
	  else
	    echo -e "\e[91;40m Nothing to remove in ${modname}${themepath} \e[0m"
	  fi
      done
    else
      echo -e "\e[91;40m Can't find theme ${themename}. \e[0m"
    fi
  else
    echo -e "\e[91;40m The path to your Horde install is wrong. \e[0m"
    echo -e "\e[91;40m Please correct this and try again. \e[0m"
  fi
}

# Thanks to various linux forums
# Function checks if array 2 contains element from array 1
function containsElement () {
  local e
  for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
  return 1
}

# Function uses containsElement to check if any of our themes is in array "installed_themes".
# We need to check in same  format because of name inconsistency
function interfering {
  count_ins="${#installed_themes[@]}" 
  for ((y=0; y<"${count_ins}"; y++ )); do
    ins_theme="${installed_themes[$y]}"
    containsElement "${ins_theme,,}" "${our_themes[@],,}"
    if [ $? == 0 ]; then
      upgrade_themes=("${upgrade_themes[@]} ${ins_theme}")
    fi
  done
}
if [ -z $2 ]; then
  interfering
# If any of our themes is installed ask which one to remove
  if [ "${#upgrade_themes[@]}" -eq 0 ]; then
    echo -e "\e[91;40m It look like you don't have any themes of ours installed. \e[0m"
  else
    echo -e "\e[91;40m You have the folowing themes installed \e[0m"
    echo -e "\e[91;40m ${upgrade_themes[@]} \e[0m"
    echo -en "\e[91;40m Which one do you want to remove?  \e[0m"
    read themename
    themepath="/themes/${themename}"
    remove
  fi
else
  themename="$2"
  themepath="/themes/${themename}"
  remove
fi

echo "------ DONE ------"